from distutils.core import setup
from glob import glob
setup(name='lios',
      version='2.5',
      description='Easy-OCR solution and Tesseract trainer for GNU/Linux',
      author='Nalin.x.Linux',
      author_email='nalin.x.Linux@gmail.com',
      url='https://gitlab.com/Nalin-x-Linux/lios-3',
      license = 'GPL-3',
      packages=['lios','lios/ocr','lios/scanner','lios/ui/','lios/ui/gtk'],
      data_files=[('share/lios/',['share/lios/readme.text','share/lios/text_cleaner_list.text','share/lios/lios.png']),
      ('share/lios/icons/',['share/lios/icons/about.png']),
      ('share/lios/icons/',['share/lios/icons/accessories-dictionary.png']),
      ('share/lios/icons/',['share/lios/icons/accessories-text-editor.png']),
      ('share/lios/icons/',['share/lios/icons/application-pdf.png']),
      ('share/lios/icons/',['share/lios/icons/audio-input-microphone.png']),
      ('share/lios/icons/',['share/lios/icons/camera.png']),
      ('share/lios/icons/',['share/lios/icons/camera-web.png']),
      ('share/lios/icons/',['share/lios/icons/check-spelling.png']),
      ('share/lios/icons/',['share/lios/icons/clear.png']),
      ('share/lios/icons/',['share/lios/icons/convert.png']),
      ('share/lios/icons/',['share/lios/icons/document-new.png']),
      ('share/lios/icons/',['share/lios/icons/document-open.png']),
      ('share/lios/icons/',['share/lios/icons/document-save.png']),
      ('share/lios/icons/',['share/lios/icons/document-save-as.png']),
      ('share/lios/icons/',['share/lios/icons/edit-copy.png']),
      ('share/lios/icons/',['share/lios/icons/edit-cut.png']),
      ('share/lios/icons/',['share/lios/icons/edit-paste.png']),
      ('share/lios/icons/',['share/lios/icons/edit-redo.png']),
      ('share/lios/icons/',['share/lios/icons/edit-undo.png']),
      ('share/lios/icons/',['share/lios/icons/emblem-favorite.png']),
      ('share/lios/icons/',['share/lios/icons/find.png']),
      ('share/lios/icons/',['share/lios/icons/find-replace.png']),
      ('share/lios/icons/',['share/lios/icons/flip-vertical.png']),
      ('share/lios/icons/',['share/lios/icons/go-down.png']),
      ('share/lios/icons/',['share/lios/icons/go-jump.png']),
      ('share/lios/icons/',['share/lios/icons/go-up.png']),
      ('share/lios/icons/',['share/lios/icons/gtk-execute.png']),
      ('share/lios/icons/',['share/lios/icons/help.png']),
      ('share/lios/icons/',['share/lios/icons/image.png']),
      ('share/lios/icons/',['share/lios/icons/info.png']),
      ('share/lios/icons/',['share/lios/icons/insert-text.png']),
      ('share/lios/icons/',['share/lios/icons/media-playback-start.png']),
      ('share/lios/icons/',['share/lios/icons/media-playback-stop.png']),
      ('share/lios/icons/',['share/lios/icons/preferences.png']),
      ('share/lios/icons/',['share/lios/icons/print.png']),
      ('share/lios/icons/',['share/lios/icons/print-preview.png']),
      ('share/lios/icons/',['share/lios/icons/process-stop.png']),
      ('share/lios/icons/',['share/lios/icons/quit.png']),
      ('share/lios/icons/',['share/lios/icons/remove.png']),
      ('share/lios/icons/',['share/lios/icons/rotate-left.png']),
      ('share/lios/icons/',['share/lios/icons/rotate-right.png']),
      ('share/lios/icons/',['share/lios/icons/scanner.png']),
      ('share/lios/icons/',['share/lios/icons/sort-descending.png']),
      ('share/lios/icons/',['share/lios/icons/system-upgrade.png']),
      ('share/lios/icons/',['share/lios/icons/zoom-fit.png']),
      ('share/lios/icons/',['share/lios/icons/zoom-in.png']),
      ('share/lios/icons/',['share/lios/icons/zoom-out.png']),
      ('share/applications/',['share/applications/Lios.desktop','share/applications/Tesseract-Trainer.desktop']),
      ('share/man/man1/',['share/man/man1/lios.1.gz','share/man/man1/train-tesseract.1.gz']),
      ('share/doc/lios/',['share/doc/lios/copyright']),('share/pixmaps/',['share/pixmaps/lios.xpm']),
      ('bin',['bin/lios','bin/train-tesseract'])]
      )
# sudo python3 setup.py install --install-data=/usr
